@@ -8,12 +8,13 @@ from timestream.api import DEFAULT_CLIENT_CONFIG, TimestreamLoader
 from timestream.dimensions import TimestreamDimensions
 from timestream.load_result import TimestreamLoadResult
 from timestream.record import RejectedTimestreamRecord, TimestreamRecord
+
+
 class TestTimestreamLoader_UnitTests:
 
     @pytest.mark.unit
     def test_load(self):
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(None)
+        mock_session = self.get_mock_session()
         loader = TimestreamLoader('test', 'test', mock_session)
         mock_session.assert_expectations()
 
@@ -37,8 +38,7 @@ class TestTimestreamLoader_UnitTests:
 
     @pytest.mark.unit
     def test_load_preview(self):
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(None)
+        mock_session = self.get_mock_session()
         loader = TimestreamLoader('test', 'test', mock_session)
         mock_session.assert_expectations()
 
@@ -61,10 +61,8 @@ class TestTimestreamLoader_UnitTests:
         assert result['error'] == None
 
     @pytest.mark.unit
-    def test_private_load(self):
-        # Test preview
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(None)
+    def test_private_load_no_data(self):
+        mock_session = self.get_mock_session()
         loader = TimestreamLoader('test', 'test', mock_session)
         mock_session.assert_expectations()
 
@@ -79,6 +77,8 @@ class TestTimestreamLoader_UnitTests:
         assert len(result.rejected_records) == 0
         assert result.error == None
 
+    @pytest.mark.unit
+    def test_private_load_insert(self):
         # Test insert
         expected_source = 'test soure'
         expected_reason = 'test reason'
@@ -88,38 +88,65 @@ class TestTimestreamLoader_UnitTests:
                 'Reason': expected_reason
             }]
         }, 'test operation')
-        expected_records=[{
+        expected_records = [{
             'MeasureName': 'valid',
-            'MeasureValue': '1.0'
+            'MeasureValue': '1.0',
+            'Dimensions': []
         }, {
             'MeasureName': 'invalid',
-            'MeasureValue': '0.0'
+            'MeasureValue': '0.0',
+            'Dimensions': []
+        }, {
+            'MeasureName': 'valid',
+            'MeasureValue': '2.0',
+            'Dimensions': [{
+                'Name': 'notshared',
+                'Value': 'not_shared_dimension',
+                'DimensionValueType': 'VARCHAR'
+            }]
+        }]
+        expected_common_dimensions = [{
+            'Name': 'shared',
+            'Value': 'shared_dimension',
+            'DimensionValueType': 'VARCHAR'
+        }, {
+            'Name': 'source',
+            'Value': 'test soure',
+            'DimensionValueType': 'VARCHAR'
         }]
         expected_capture_time = '1609459200000'
 
         mock_client = MockApi()
-        mock_client.on('write_records',
+        mock_client.on(
+            'write_records',
             DatabaseName='test',
             TableName='test',
             CommonAttributes={
-                'Dimensions': [{'Name': 'source', 'Value': expected_source, 'DimensionValueType': 'VARCHAR'}],
+                'Dimensions': expected_common_dimensions,
                 'MeasureValueType': 'DOUBLE',
                 'Time': expected_capture_time,
                 'Version': 1
             },
             Records=expected_records
         ).raise_exception(expected_error)
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
-        
-        loader = TimestreamLoader('test', 'test', mock_session)  
+        mock_session = self.get_mock_session(mock_client)
+
+        loader = TimestreamLoader('test', 'test', mock_session)
         result = loader._TimestreamLoader__load(
-            data=[{'valid': 1.0, 'invalid': 0.0}],
+            data=[{
+                'valid': 1.0,
+                'invalid': 0.0,
+                'shared': 'shared_dimension'
+            }, {
+                'valid': 2.0,
+                'shared': 'shared_dimension',
+                'notshared': 'not_shared_dimension'
+            }],
             capture_time=datetime(2021, 1, 1, 0, 0, 0, 0, pytz.UTC),
             source=expected_source
         )
         assert type(result) == TimestreamLoadResult
-        assert len(result.accepted_records) == 1
+        assert len(result.accepted_records) == 2
         assert result.accepted_records[0].measure_name == 'valid'
         assert len(result.rejected_records) == 1
         rejected_record = result.rejected_records[0]
@@ -129,18 +156,20 @@ class TestTimestreamLoader_UnitTests:
         mock_session.assert_expectations()
         mock_client.assert_expectations()
 
+    @pytest.mark.unit
+    def test_private_load_upsert(self):
         # Test upsert
-        expected_records=[{
+        expected_capture_time = '1609459200000'
+        expected_records = [{
             'MeasureName': 'valid',
             'MeasureValue': '1.0',
-            'Time': expected_capture_time
-        }, {
-            'MeasureName': 'invalid',
-            'MeasureValue': '0.0',
+            'Dimensions': [],
             'Time': expected_capture_time
         }]
+
         mock_client = MockApi()
-        mock_client.on('write_records',
+        mock_client.on(
+            'write_records',
             DatabaseName='test',
             TableName='test',
             CommonAttributes={
@@ -149,24 +178,28 @@ class TestTimestreamLoader_UnitTests:
                 'Version': int(expected_capture_time)
             },
             Records=expected_records
-        ).raise_exception(expected_error)
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
-        
-        loader = TimestreamLoader('test', 'test', mock_session, current_time_ms=int(expected_capture_time))
+        ).return_val(None)
+        mock_session = self.get_mock_session(mock_client)
+
+        loader = TimestreamLoader(
+            'test',
+            'test',
+            mock_session,
+            current_time_ms=int(expected_capture_time)
+        )
         result = loader._TimestreamLoader__load(
-            data=[{'valid': 1.0, 'invalid': 0.0, 'time': int(expected_capture_time), 'version': 2}],
+            data=[{
+                'valid': 1.0,
+                'time': int(expected_capture_time),
+                'version': 2
+            }],
             capture_time=datetime(2021, 1, 1, 0, 0, 0, 0, pytz.UTC),
             upsert=True
         )
         assert type(result) == TimestreamLoadResult
         assert len(result.accepted_records) == 1
         assert result.accepted_records[0].measure_name == 'valid'
-        assert len(result.rejected_records) == 1
-        rejected_record = result.rejected_records[0]
-        assert type(rejected_record) == RejectedTimestreamRecord
-        assert str(rejected_record.error) == expected_reason
-        assert rejected_record.record.measure_name == 'invalid'
+        assert len(result.rejected_records) == 0
         mock_session.assert_expectations()
         mock_client.assert_expectations()
 
@@ -182,18 +215,21 @@ class TestTimestreamLoader_UnitTests:
         }, 'test operation')
         expected_valid_record = TimestreamRecord('valid', 1.0)
         expected_invalid_record = TimestreamRecord('invalid', 0.0)
-        expected_records=[{
+        expected_records = [{
             'MeasureName': 'valid',
-            'MeasureValue': '1.0'
+            'MeasureValue': '1.0',
+            'Dimensions': []
         }, {
             'MeasureName': 'invalid',
-            'MeasureValue': '0.0'
+            'MeasureValue': '0.0',
+            'Dimensions': []
         }]
         expected_capture_time = '1609459200000'
-        
+
         # Set up mocks
         mock_client = MockApi()
-        mock_client.on('write_records',
+        mock_client.on(
+            'write_records',
             DatabaseName='test',
             TableName='test',
             CommonAttributes={
@@ -204,13 +240,12 @@ class TestTimestreamLoader_UnitTests:
             },
             Records=expected_records
         ).raise_exception(expected_error)
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
-        
+        mock_session = self.get_mock_session(mock_client)
+
         # Test ClientError occurs during Timestream write
         loader = TimestreamLoader('test', 'test', mock_session)
         accepted_records, rejected_records = loader._TimestreamLoader__write_insert_records(
-            dimensions=TimestreamDimensions(),
+            common_dimensions=TimestreamDimensions(),
             records=[expected_valid_record, expected_invalid_record],
             capture_time=datetime(2021, 1, 1, 0, 0, 0, 0, tzinfo=pytz.UTC)
         )
@@ -229,7 +264,7 @@ class TestTimestreamLoader_UnitTests:
     @pytest.mark.unit
     def test_write_upsert_records(self):
         # Set expectations
-        expected_version=1609459200000
+        expected_version = 1609459200000
         expected_reason = 'test reason'
         expected_error = ClientError({
             'RejectedRecords': [{
@@ -239,17 +274,20 @@ class TestTimestreamLoader_UnitTests:
         }, 'test operation')
         expected_valid_record = TimestreamRecord('valid', 1.0)
         expected_invalid_record = TimestreamRecord('invalid', 0.0)
-        expected_records=[{
+        expected_records = [{
             'MeasureName': 'valid',
-            'MeasureValue': '1.0'
+            'MeasureValue': '1.0',
+            'Dimensions': []
         }, {
             'MeasureName': 'invalid',
-            'MeasureValue': '0.0'
+            'MeasureValue': '0.0',
+            'Dimensions': []
         }]
-        
+
         # Set up mocks
         mock_client = MockApi()
-        mock_client.on('write_records',
+        mock_client.on(
+            'write_records',
             DatabaseName='test',
             TableName='test',
             CommonAttributes={
@@ -259,13 +297,17 @@ class TestTimestreamLoader_UnitTests:
             },
             Records=expected_records
         ).raise_exception(expected_error)
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
-        
+        mock_session = self.get_mock_session(mock_client)
+
         # Test ClientError occurs during Timestream write
-        loader = TimestreamLoader('test', 'test', mock_session, current_time_ms=expected_version)
+        loader = TimestreamLoader(
+            'test',
+            'test',
+            mock_session,
+            current_time_ms=expected_version
+        )
         accepted_records, rejected_records = loader._TimestreamLoader__write_upsert_records(
-            dimensions=TimestreamDimensions(),
+            common_dimensions=TimestreamDimensions(),
             records=[expected_valid_record, expected_invalid_record]
         )
         assert len(accepted_records) == 1
@@ -282,8 +324,7 @@ class TestTimestreamLoader_UnitTests:
 
     @pytest.mark.unit
     def test_process_client_error(self):
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(None)
+        mock_session = self.get_mock_session()
         loader = TimestreamLoader('test', 'test', mock_session)
         mock_session.assert_expectations()
 
@@ -312,8 +353,7 @@ class TestTimestreamLoader_UnitTests:
 
     @pytest.mark.unit
     def test_parse_data(self):
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(None)
+        mock_session = self.get_mock_session()
         loader = TimestreamLoader('test', 'test', mock_session)
 
         # Test no data
@@ -352,8 +392,7 @@ class TestTimestreamLoader_UnitTests:
 
     @pytest.mark.unit
     def test_parse_item(self):
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(None)
+        mock_session = self.get_mock_session()
         loader = TimestreamLoader('test', 'test', mock_session)
         mock_session.assert_expectations()
 
@@ -414,29 +453,34 @@ class TestTimestreamLoader_UnitTests:
         assert record.measure_name == 'integer'
         assert record.measure_value == 10.0
         assert record.mstime == 1609459200000
-        assert record.time == datetime.combine(date(2021, 1, 1), datetime.min.time()).replace(tzinfo=pytz.UTC)
+        assert record.time == datetime.combine(
+            date(2021, 1, 1),
+            datetime.min.time()
+        ).replace(tzinfo=pytz.UTC)
 
         # Test parse error
         with pytest.raises(AssertionError) as e_info:
             loader._TimestreamLoader__parse_item({
-                'unparseable': {'key': 'value'} 
+                'unparseable': {'key': 'value'}
             }, False, False)
         assert str(e_info.value) == 'Could not create a record or dimension from type "<class \'dict\'>" at key unparseable'
 
         # Test ignore parse error
         records = loader._TimestreamLoader__parse_item({
-            'unparseable': {'key': 'value'} 
+            'unparseable': {'key': 'value'}
         }, False, True)
         assert len(records) == 0
 
     @pytest.mark.unit
     def test_dt_to_ms(self):
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(None)
+        mock_session = self.get_mock_session()
         loader = TimestreamLoader('test', 'test', mock_session)
         mock_session.assert_expectations()
-        
-        dt = datetime.combine(date(2021, 1, 1), datetime.min.time()).replace(tzinfo=pytz.UTC)
+
+        dt = datetime.combine(
+            date(2021, 1, 1),
+            datetime.min.time()
+        ).replace(tzinfo=pytz.UTC)
         assert loader._TimestreamLoader__dt_to_ms(dt) == 1609459200000
         assert loader._TimestreamLoader__dt_to_ms(None) == -1
 
@@ -456,8 +500,7 @@ class TestTimestreamLoader_UnitTests:
             'describe_database',
             DatabaseName='test',
         ).return_val(mock_response)
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
+        mock_session = self.get_mock_session(mock_client)
 
         # Test False when using TimestreamLoader's database attribtue
         loader = TimestreamLoader('default', 'default', mock_session)
@@ -493,8 +536,7 @@ class TestTimestreamLoader_UnitTests:
             DatabaseName='test',
             TableName='test'
         ).return_val(mock_response)
-        mock_session = MockApi()
-        mock_session.on('client', 'timestream-write', config=DEFAULT_CLIENT_CONFIG).return_val(mock_client)
+        mock_session = self.get_mock_session(mock_client)
 
         # Test False when using TimestreamLoader's database and table attribtue
         loader = TimestreamLoader('default', 'default', mock_session)
@@ -511,3 +553,12 @@ class TestTimestreamLoader_UnitTests:
 
         mock_session.assert_expectations()
         mock_client.assert_expectations()
+
+    def get_mock_session(self, return_val=None) -> MockApi:
+        mock_session = MockApi()
+        mock_session.on(
+            'client',
+            'timestream-write',
+            config=DEFAULT_CLIENT_CONFIG
+        ).return_val(return_val)
+        return mock_session
